% function [outrays,outvals,outwghts,matches]=LapBunny(numiterations,plotmth)
% -------------------------------------------------------
% Does the set up work for characterizing the laplacian on the Rabbit
% corresponding to the conformal measure.
%
%
% level speficies the number of times to apply the polynomial
% plotmth(2) is a character
%      -'h'-height: plots measure in 3D
%      -'b'-bubble: plots measure in 2D
%      -'d'- deriv: plots 2|z|
%      -'u'- nu: plot nu against ray angle 
%      -'c'- cummulative distribution function 
%      -'r'-raycheck:plots exterior rays as height
%      -'a'- all:  spits out all plots 
%      -'n'- none(default)turns off plot, useful as often a helper func.
%   if you would like to specify two of the three plot methods feel free
%   to do so 
%
% Example [r,v,w]=LapBunny(5,'br']); 


function [outrays,outvals,outwghts,matches]=LapBunny(level,plotmth) 

if(nargin==1)
    plotmth='n';
end

c=-.122+.745*sqrt(-1);
alpha=1.39; % Better Estimate??

numpoints = 3*2^(level);

rays    = zeros(2, numpoints); % External ray values
values  = zeros(2, numpoints); % Point values
weights = zeros(2, numpoints); % Given by the conformal measure
imgindx  = zeros(2, numpoints); % Records the index of the image of a point

matches  = zeros(numpoints/3,3);  % records the match values for the final rays matrix

% We need a method to sort ray angles:  draw a line through the junction
% points of the bulbs at each level, draw a 'line' that in the limit would
% join ray angle 0 and 1/2.  Note that this line is z->-z symmetric so we
% need only half the line.  Note that this line will be generated by
% successive images of the right most point

center =  zeros(1,level-1); %r
% Set values for the 0th iteration
if (level>=0)
    rays(1,1:3)   = [1 2 4];  % really just numerator 
    values(1,1)   = min(roots([1 -1 c]));
    values(1,2)   = values(1,1); 
    values(1,3)   = values(1,1); 
    weights(1,1:3)= [1,1,1];  % yep, this is arbitrary (but they must be equal)
    matches(1,:)= [1,2,3];  % these points match
else
    error('Sorry cannot go backward-anywise it would be rather boring, the images of a fixed point....')
end 

% 1st iteration, it's unique one of the preimages is a duplicate
if ( level>=1 )
    values(2,:)  = values(1,:);
    values(2,4:6)= min(roots([1 0 c-values(1,1)]))*ones(1,3);
    
    rays(2,:)  = 2*rays(1,:);
    rays(2,4)  = 1; % 'half' of each old angle
    rays(2,5)  = 9; % 'half' of each old angle 
    rays(2,6)  = 11;
    
    imgindx(2,1:6) = ones(1,6);  % at this level every point maps to the first
    weights(2,1:6) = weights(1,imgindx(2,1:6)).*abs(2*values(2,1:6)).^(-alpha);
   
    matches(2,:)=[4,5,6];
    
    % Update Arrays
    values(1,:) = values (2,:);
    weights(1,:)= weights(2,:);
    imgindx(1,:) = imgindx (2,:);
    rays(1,:)   = rays   (2,:); 
    center(1) = values(1,6);

    
elseif (level==0)
    fprintf('Really, you dont want me to iterate?\r')
else 
    error('Out of curiousity, what would a fractional interation be?')
end

% More iterations 
% Some facts that make the code make more sense
%     + iteration i there are 3*2^(i) points 
%     + at each level the donominator value is 7*2^(i)

for i=2:level
    tic
% Preserve old values
 %   values(2,:) = values(1,:); Redundant lines
 %   imgindx(2,:) = imgindx(1,:);
    rays(2,:) = 2*rays(1,:);
 	
% Calculate New Values   
    newval = [sqrt(values(1,3*2^(i-2)+1:3*2^(i-1))-c),...
                -sqrt(values(1,3*2^(i-2)+1:3*2^(i-1))-c)]; 
 
    checkpts = real(newval)>real(center(i-1));
    [maxx,inx] =  max(imag(newval(checkpts)));
 
    center(i)= newval(inx);  % take the new value with the largest real part
                             % breaks down after level 8.  
                           
    safetynet =center(i)+(.01:.01:.05);
    centerline=[-safetynet(5:-1:1),-center(i:-1:1),center(1:i),safetynet];
    [testers,ibb]=connectdots(centerline,newval);
        
    helper=imag(newval)>testers;  % get points above the line 
    helper(ibb)=~helper(ibb);
    
    helper=logical(helper.*(newval~=center(i)&newval~=-center(i)));
    % for the center line with positive x assign one small angle value
    
    px = (newval==center(i)).*(mod(1:length(newval),3)==1);
    % for the center line with negative x assign two small angle values 
    nx = (newval==-center(i)).*(mod(1:length(newval),3)~=1);
    helper= helper+px+nx; 
       
    newray = (helper).*[rays(1,3*2^(i-2)+1:3*2^(i-1)),rays(1,3*2^(i-2)+1:3*2^(i-1))];
    newray = newray + (~helper).*[rays(1,3*2^(i-2)+1:3*2^(i-1))+7*2^(i-1),...
                                          rays(1,3*2^(i-2)+1:3*2^(i-1))+7*2^(i-1)];
    
    % Put it together    
    values(2,3*2^(i-1)+1:3*2^(i)) = newval;
    rays(2,3*2^(i-1)+1:3*2^(i))   = newray;
    imgindx(2,3*2^(i-1)+1:3*2^(i)) = [3*2^(i-2)+1:3*2^(i-1),3*2^(i-2)+1:3*2^(i-1)];
    weights(2,1:3*2^(i)) = weights(1,imgindx(2,1:3*2^(i))).*((2*abs(values(2,1:3*2^(i)))).^(-alpha));

    % Update Arrays
    values(1,:) = values (2,:);
    weights(1,:)= weights(2,:);
    imgindx(1,:) = imgindx (2,:);
    rays(1,:)   = rays   (2,:); 
    fprintf('Iteration %d ',i);
    toc
end

% Sort by ray order, tidy up and find matches
outrays    = rays(2,:);
[outrays, I] = sort(outrays); 
outvals  = values(2,I);
outwghts = weights(2,I)/sum(weights(2,:));

% Calculate Matches (Brute force...) 
unused=1:numpoints; %store unused indices
for i=1:numpoints/3  % evaluate matches for half the points
    % calculate match at first unused index
    matchindx    = find(outvals==outvals(min(unused)));
    matches(i,:) = matchindx;
    unused(matchindx)=(numpoints+1)*ones(1,3);
end    

if (plotmth=='n')
    return;
end

for i=1:length(plotmth)
    if (plotmth(i)=='h'||plotmth(i)=='a') 
        % Height plot
        % uses the measure as the height
        scatter3(real(outvals),imag(outvals),outwghts,4,outwghts,'*')
        prettyify;
        colorbar;
        title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='b'||plotmth(i)=='a') 
        % Bubble Plot
        % use measures as the size of the weight
        figure;
        scatter(real(outvals),imag(outvals),8*numpoints*outwghts,outwghts,'filled');
        prettyify
        axis equal
        %title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='d'||plotmth(i)=='a') 
        % Derivative Plot
        % use 2|z| as the size of the weight
        figure;
        scatter(real(outvals),imag(outvals),25*abs(outvals),abs(outvals),'filled');
        prettyify
        axis equal
        % title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='c'||plotmth(i)=='a') 
        % Nu Plot
        % cummulative distribution function
        figure;
        plot(outrays/max(outrays+1),cumsum(outwghts),'k');
        ylim([0,1])
        % title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='u'||plotmth(i)=='a') 
        % Nu Plot
        % cummulative distribution function
        figure;
        plot(outrays/max(outrays+1),outwghts,'k');
        % title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='r'||plotmth(i)=='a') 
        % Check rays
        % plot rays as height, if rays are computed incorrectly 
        % you can expect to see points connected to the opposite side of
        % the quasicircle--can happen on asymmetric quasicircles
        figure;
        plot3(real(outvals),imag(outvals), outrays)
        title('Exterior ray angle check')
    end
    
end