% function [out, ibb]=connectdots(data, inp) 
% -----------------------------------------------------
% This function is a helper function for LapBunny.m that assists in the
% determination of ray angles.  

% data is the list of points on 'center line' 
% inp is the set of point whose ray angles need to be determined 

function [out,ibb]=connectdots(data,inp) 
% The basic of our method for determining the ray angles is to draw a line
% through the center of each bulb that contains points with both positive
% and negative angles, we call the piecewise linear relation thus defined
% the center 'line'.  Points 'above' this line will have positive angles,
% points bellow this angle with have negative angles. 
% 
% This function helps in the determination of ray angles by working to
% straighten the center 'line' so that we can use the built in interpolation
% function to predict.
% 
% Where necessary records points whose values will be switched by the
% method are recorded so that they can be returned to their correct
% values.

% Taryn Flock, Summer 2008, Cornell REU

xdata=real(data);
ydata=imag(data);
xin=real(inp);
yin=imag(inp);

xdatap=xdata;

i=2; 
while (xdatap(i)<0)
    %fprintf('inwhile\r')
    if (xdatap(i)<xdatap(i-1)||xdatap(i)>xdatap(i+1)) 
            xdatap(i)=(xdatap(i-1)+xdatap(i+1))/2;
    end
    i=i+1;
end

i=length(xdatap)-1;
while (xdatap(i)>0)
   % fprintf('inwhile\r')
    if (xdatap(i)<xdatap(i-1)||xdatap(i)>xdatap(i+1)) 
            xdatap(i)=(xdatap(i-1)+xdatap(i+1))/2;
    end
    i=i-1;
end



% figure;
% hold all
% plot(xdata,ydata,'.')
% plot(xdatap,ydata,'*')

out=interp1(xdatap,ydata,xin,'linear');  

if (sum(xdata~=xdatap&xdata<0)&&sum(xdata(xdata~=xdatap&xdata>0)))
    minx=min(xdata(xdata~=xdatap&xdata<0));
    maxx=max(xdata(xdata~=xdatap&xdata<0));
   
    miny=min(ydata(xdata~=xdatap&xdata<0));
    maxy=max(ydata(xdata~=xdatap&xdata<0));
  

    minx=min(xdata(xdata~=xdatap&xdata>0));
    maxx=max(xdata(xdata~=xdatap&xdata>0));
   
    miny=min(ydata(xdata~=xdatap&xdata>0));
    maxy=max(ydata(xdata~=xdatap&xdata>0));
    
end

%--- Brute Force Fixes---% 
ibb=(xin<=-1.2791+.0001).*(xin>=-1.2791-.0001);
ibb=ibb.*(yin<=0.4812+.0001).*(yin>=0.4812-.0001);
ibb2=(xin<=1.2791+.0001).*(xin>=1.2791-.0001);
ibb2=ibb2.*(yin<=-0.4812+.0001).*(yin>=-0.4812-.0001);


ibb3=(xin<=-1.27729+.00001).*(xin>=-1.27729-.00001);
ibb3=ibb3.*(yin<=0.47998+.00001).*(yin>=0.47998-.00001);
ibb4=(xin<=1.27729+.00001).*(xin>=1.27729-.00001);
ibb4=ibb4.*(yin<=-0.4800+.0001).*(yin>=-0.47998-.00001);

%--- good 11 only---%
ibb5=(abs(xin)<=1.2778).*(abs(xin)>=1.2777);
ibb6=(abs(xin)<=1.2773).*(abs(xin)>=1.2772);
ibb6=ibb6.*(abs(yin)<=0.4799).*(abs(yin)>=0.4798);
ibb7=(abs(xin)<=1.2766).*(abs(xin)>=1.2765);
ibb=logical(ibb+ibb2+ibb3+ibb4+ibb5+ibb6+ibb7);


sum(ibb);

   